'use strict';

angular.module('adf.widget.electre.lavoisier')
        .directive('parametersView',
            [
                "lavoisierHttpRequestService", "parserService", "domService", "$compile",
                function (lavoisierHttpRequestService, parserService, domService, $compile) {
                    /**
                     * Donne les éléments de formulaire contenu dans l'élément donné.
                     * @param {Element} domElement
                     * @returns {Element}
                     */
                    function getFormElement (domElement) {
                        if (domElement.getElementsByTagName("select").length > 0) return domElement.getElementsByTagName("select")[0];
                        else if (domElement.getElementsByTagName("input").length > 0) return domElement.getElementsByTagName("input")[0];
                    }

                    function updateParametersView (scope, element, attrs) {
                        // Suppression des node fils de 'element'.
                        if (element.children().length > 0) element.empty();

                        // Préparation de la configuration pour acceptation des valeur données.
                        if (!scope.config["parameters_" + scope.config.view]) {
                            scope.config["parameters_" + scope.config.view] = {};
                        }
                        // Construction de l'url de récupération de la page HTML.
                        var url_for_parameters = scope.config.baseurl + "/form/" + scope.config.view;

                        lavoisierHttpRequestService.getResult({url : url_for_parameters, params : {accept : "html"}})
                            .then(function (value) {

                                // Convertion de la valeur de retour en HTML DOM.
                                var html = parserService.htmlDomFromString(value);
                                // Récupération du formulaire qui nous interresse.
                                var parameter_form = html.getElementsByClassName("content-wrapper")[0].getElementsByTagName("form")[0];
                                // Suppression des deux dernier node enfant du formulaire.
                                for (var i = 1 ; i <= 2 ; i++) parameter_form.removeChild(parameter_form.lastChild);
                                // Récupération de tous les div contenant les éléments de formulaire.
                                var div_form_group = parameter_form.getElementsByClassName("form-group");
                                // Récupération de tous les éléments de formulaire et conversion pour intégration.
                                for (var i_div = 0 ; i_div < div_form_group.length ; i_div++) {
                                    // Récupération de l'élément de formulaire.
                                    var formElement = getFormElement(div_form_group[i_div]);
                                    // Création du div qui va contenir l'élément de formulaire.
                                    var div = domService.createElement("div")
                                            .setAttribute("class", "form-group")
                                            .getDom();
                                    // Création et ajout du label.
                                    var label = domService.createElement("label").setAttribute("for", formElement.getAttribute("name")).getDom();
                                    label.innerHTML = formElement.getAttribute("name");
                                    div.appendChild(label);
                                    // Modification de l'élément de formulaire et ajout dans div.
                                    scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] = scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] || undefined;
                                    div.appendChild(
                                        domService.setDom(formElement)
                                            .setAttribute("class", "form-control")
                                            .setAttribute("ng-model", "config['parameters_" + scope.config.view + "']['" + formElement.getAttribute("name") + "']")
                                            .getDom());

                                    // Récupération de la valeur par défaut de l'élément s'il en a une.
                                    if (!scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")]) {
                                        // Traitement différent selon si input ou select
                                        if (formElement.tagName === "input") {
                                            scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] = formElement.value || undefined;
                                        } else {
                                            var option_tab = formElement.getElementsByTagName("option");
                                            for (var o = 0 ; o < option_tab.length ; o++) {
                                                if (option_tab[o].getAttribute("selected") !== null) {
                                                    scope.config["parameters_" + scope.config.view][formElement.getAttribute("name")] = option_tab[o].value;
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                    // Ajout des éléments dans l'élément de directive.
                                    element.append($compile(div)(scope));

                                    // Ajout ici de l'évènement car si on l'ajoute avant le deferred, on risque de l'ajouter avant que l'élément "#select_for_lavoisier_view" ne soit lui même ajouter dans le html.
                                    if (!scope.control.controller.eventAdded) {
                                        // Ajout d'un évènement sur l'élément select des vues.
                                        angular.element(document.getElementById("select_for_lavoisier_view")).on("change", scope.control.controller.majParametersContent);
                                        // Pour n'ajouter l'évènement qu'une foi.
                                        scope.control.controller.eventAdded = true;
                                    }
                                }
                            });
                    }

                    return {
                        scope : {
                            control : '='
                        },
                        link : function (scope, element, attrs) {
                            scope.config = scope.control.config;
                            scope.control.controller.majParametersContent = function () { updateParametersView(scope, element, attrs); };
                            scope.control.controller.majParametersContent();
                        }
                    };
                }
            ]
        );